/*
 *   cat-meter
 *   Copyright (C) 2004  Josef Kufner <jkufner@seznam.cz>
 *   http://jk.myserver.cz/programy/cat-meter
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>

#define BUF_SIZE 	1024
#define AVG		5
#define IN_FD		0
#define OUT_FD		1

long int counter = 0;
long int avg_buf[AVG] = { 0, 0, 0, 0, 0 };
int avg_pos = 0;

int first_time = 1;

void on_alarm(int sig) {
	float rate, avg_rate, avg_sum = 0;
	int i;
	
	/* current */
	rate = counter / 1024.0;

	/* average */
	if (first_time) {
		first_time = 0;
		for (i = 0; i < AVG; i++) {
			avg_buf[i] = counter;
		}
	} else {
		avg_buf[avg_pos] = counter;
		avg_pos = (avg_pos + 1) % AVG;
	}
	
	for (i = 0; i < AVG; i++) {
		avg_sum += avg_buf[i];
	}
	avg_rate = (float) avg_sum / (float) AVG / 1024.0;
	
	fprintf(stderr, "\r%5.3f KBytes/sec  "
			"(%d sec avg: %5.3f Kbytes/sec)  ",
			rate, AVG, avg_rate);
	counter = 0;
	alarm(1);
}

int main (void) {
	ssize_t bytes = 0;
	unsigned char data[BUF_SIZE];
	
	signal(SIGALRM, on_alarm);
	alarm(1);
	
	for (;;) {
		if ((bytes = read(IN_FD, data, BUF_SIZE)) > 0) {
			write(OUT_FD, data, bytes);
			counter += bytes;
		} else if (errno != EINTR) {
			break;
		}
	}
	return 0;
}

